#!/usr/bin/env python
# -*- mode: python; coding: koi8-r; -*-
import os, locale, string

program_name = 'PyBookReader'
program_version = '0.4.7'

##----------------------------------------------------------------------
## configuration

config_dir = os.path.expanduser('~/.pybr')
config_file = os.path.expanduser('~/.pybr/config')
bookmarks_file = os.path.expanduser('~/.pybr/bookmarks')
filetypes_file = os.path.expanduser('~/.pybr/filetypes')
dictclient_rc_file =  os.path.expanduser('~/.pybr/dictclient.rc')

charsets_list = ('koi8-r', 'cp1251', 'cp866', 'iso8859-5', 'utf-8')

default_description_length = 200
preview_length = 4000
default_encoding = locale.getlocale()[1]
default_category = 'Default'

##----------------------------------------------------------------------
## filetypes

from parsers import *

file_types = {

    'gzipped_plain_text' : (('.gz',),                  # suffixes
                            gzipped_plain_text_parser, # parser
                            None, None),          # external_command, encoding

    'html'               : (('.htm', '.html'),
                            html_parser,
                            None, None),

##     'lib_ru_parser'        : (('txt.html',),
##                             lib_ru_parser,
##                             None, None),

    'gzipped_html'       : (('.htm.gz', '.html.gz'),
                            gzipped_html_parser,
                            None, None),

##     'zip'                : (('.zip'),
##                             zip_parser,
##                             None, None),

    'fb2'                : (('.fb2',),
                            fb2_parser,
                            None, None),

    'ms_doc'             : (('.doc',),
                            plain_text_parser,
                            'catdoc', None),

    'pdf'                : (('.pdf',),
                            plain_text_parser,
                            'pdftotext -enc UTF-8 %s -', 'utf-8'),

    'plain_text'         : (('',),
                            plain_text_parser,
                            None, None),
    }


def parse_filetypes():
    if not os.path.exists(filetypes_file): return
    import parsers
    try:
        fd = open(filetypes_file)
    except IOError, err:
        print >> sys.stderr, 'ERROR: Can\'t open config file: %s: %s' \
              % (filetypes_file, err[1])
        return

    new_file_types = {}

    #from pprint import pprint
    #pprint(file_types)

    def print_error(ln, msg):
        print >> sys.stderr, \
              'ERROR in config file: %s: line %d: %s' \
              % (filetypes_file, ln, msg)

    line_num = 1
    for line in fd.readlines():

        # split line
        words_list = []
        word = ''
        i = 0
        while i < len(line):
            char = line[i]
            if char == '#':
                if word:
                    print_error(line_num, 'sintax error')
                    return
                break
            if char in string.whitespace:
                if word:
                    words_list.append(word)
                    word = ''
                i += 1
                continue
            if char in ['\'', '"']:
                if word:
                    print_error(line_num, 'sintax error')
                    return
                i += 1
                j = line[i:].find(char)
                if j < 0:
                    print_error(line_num, 'no match')
                    return
                else:
                    words_list.append(line[i:i+j])
                    i += j+1
                continue
            word += char
            i += 1

        # add new file type to temporary filetypes dict
        if words_list:
            if not 3 <= len(words_list) <= 5:
                print_error(line_num, 'number of fields')
                return
            else:
                name = words_list[0]
                suffixes = words_list[1].split() or ['']
                if hasattr(parsers, words_list[2]):
                    parser = getattr(parsers, words_list[2])
                else:
                    print_error(line_num ,
                                'unknown parser: "%s"' % words_list[2])
                    return

                if len(words_list) < 4:
                    command = None
                elif words_list[3] in ['None', '']:
                    command = None
                else:
                    command = words_list[3]
                    if command.find('%s') > 0:
                        try:
                            command % ''
                        except:
                            print_error(line_num ,
                                        'format error: "%s"' % words_list[3])
                            return

                if len(words_list) < 5:
                    encoding = None
                elif words_list[4] in ['None', '']:
                    encoding = None
                else:
                    encoding = words_list[4]

                new_file_types[name] = (suffixes, parser, command, encoding)

        line_num += 1

    #pprint(new_file_types)

    # update filetypes
    global file_types
    file_types.update(new_file_types)

##----------------------------------------------------------------------
## options

def parse_options():
    pass

